#!/usr/bin/perl
#
# httpantivirus.cgi for Endian Firewall

#
#        +-----------------------------------------------------------------------------+
#        | Endian Firewall                                                             |
#        +-----------------------------------------------------------------------------+
#        | Copyright (c) 2005-2006 Endian                                              |
#        |         Endian GmbH/Srl                                                     |
#        |         Bergweg 41 Via Monte                                                |
#        |         39057 Eppan/Appiano                                                 |
#        |         ITALIEN/ITALIA                                                      |
#        |         info@endian.it                                                      |
#        |                                                                             |
#        | This program is free software; you can redistribute it and/or               |
#        | modify it under the terms of the GNU General Public License                 |
#        | as published by the Free Software Foundation; either version 2              |
#        | of the License, or (at your option) any later version.                      |
#        |                                                                             |
#        | This program is distributed in the hope that it will be useful,             |
#        | but WITHOUT ANY WARRANTY; without even the implied warranty of              |
#        | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               |
#        | GNU General Public License for more details.                                |
#        |                                                                             |
#        | You should have received a copy of the GNU General Public License           |
#        | along with this program; if not, write to the Free Software                 |
#        | Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. |
#        | http://www.fsf.org/                                                         |
#        +-----------------------------------------------------------------------------+
#

my $CONFIG_ROOT='/var/efw';
require $CONFIG_ROOT.'/header.pl';

my $conffile_default = $CONFIG_ROOT.'/havp/default/settings';
my $conffile = $CONFIG_ROOT.'/havp/settings';
my $whitelistfile = $CONFIG_ROOT.'/havp/whitelist';
my $whitelist = "";

my $lastupdate = `tac /var/log/clamav/clamd.log| grep -m1 "Database updated" | awk {' print $2" "$1" "$3" "$8" "$9 '}`;

my $do_reload = 0;

my %conf_hash = ();
my $conf = \%conf_hash;

my %checked     = ( 0 => '', 1 => 'checked', 'on' => 'checked');

my $restart = '/usr/local/bin/restartsquid';

my $enabledfile = '/var/efw/havp/enable';

sub toggle_file($$) {
    my $file = shift;
    my $set = shift;

    if ($set) {
	`touch $file`;
	return 1;
    }
    if (-e $file) {
	unlink($file);
    }
    return 0;
}


sub loadconfig() {
    if (-e $conffile_default) {
	readhash($conffile_default, $conf);
    }
    if (-e $conffile) {
	readhash($conffile, $conf);
    }
    if (-e $whitelistfile) {
	open(FILE, $whitelistfile);
	$whitelist = join("\n", <FILE>);
	close(FILE);
    }
}


sub save() {
    my %par;
    getcgihash(\%par);

    if ($par{'action'} ne 'save') {
	return (0,'');
    }

    if ($par{'MAXSCANSIZE'} ne $conf->{'MAXSCANSIZE'} or
	 ($par{'WHITELIST'} ne $whitelist)
	) {
	if ($par{'MAXSCANSIZE'} !~ /\d+/) {
	    return (1,$tr{'invalid filesize'});
	}
	$conf->{'MAXSCANSIZE'}=$par{'MAXSCANSIZE'};
	$whitelist = $par{'WHITELIST'};
	writehash($conffile, $conf);
        open (OUT, ">$whitelistfile");
        print OUT $whitelist;
        close OUT;
	$do_reload = 1;
    }
    if ($do_reload) {
        `$restart`;
    }
    return (0,'');
}

sub display() {
    if (-e '/etc/FLASH') {
	$conf->{'MAXSCANSIZE'} = 20;
	$fixed = 'DISABLED';
    }

    openbox('100%', 'left', $tr{'http antivirus'});
    print <<EOF
<FORM ACTION="" METHOD="post">
  <table border='0' cellspacing="0" cellpadding="4">
    <TR>
      <TD width="20%">$tr{'max filesize'}</TD>
      <TD align="left">
        <SELECT NAME="MAXSCANSIZE" $fixed>
EOF
;
    my @entries = qw '0 1 2 5 10 15 20 25 30 40 50 80 100 150 200 500';
    foreach my $entry (@entries) {
	my $selected = '';
	if ($entry eq $conf->{'MAXSCANSIZE'}) {
	    $selected = 'selected';
	}
	print "<OPTION $selected VALUE=\"$entry\">$entry</OPTION>";
    }
    print <<EOF
        </SELECT>
      </TD>
EOF
;

if ( $lastupdate ) {
print <<EOF
<td valign="top">
<table bgcolor="#cccccc" border="0" cellpadding="5" cellspacing="1">
    <tbody><tr>
	<td colspan="2" bgcolor="#eeeeee"><b>$tr{'Last Update'}</b></td>
    </tr>
    <tr>
      <td bgcolor='#fefefe'>
EOF
;

system('tac /var/log/clamav/clamd.log| grep -m1 "Database updated" | awk {\' print $2" "$1" "$3" "$8" "$9 \'}');

print <<EOF
      </td>
    </tr>
</tbody></table>
EOF
;
}

print <<EOF
    </TR>
    <tr>
      <td width="20%">$tr{'do not scan'}:<br>
      <textarea name='WHITELIST' cols='20' rows='4' wrap='off'>$whitelist</textarea></TD>
    </tr>
EOF
;
if (! -e $enabledfile) {
    print <<EOF
    <TR>
      <td colspan="3">
        $tr{'httpantivirus disabled'}
      </td>
    </TR>
EOF
;
}
    print <<EOF
    <TR>
      <TD>
        <INPUT TYPE='submit' NAME='submit' VALUE='$tr{save}' />
      </TD>
      <TD>&nbsp;</TD>
    </TR>
    <INPUT TYPE="hidden" NAME="action" VALUE="save">
  <TABLE>
</FORM>
EOF
;
    closebox();
    
}

sub display_error($$) {
    my $error = shift;
    my $errormessage = shift;
    if ($errormessage !~ /^$/) {
	&openbox('100%', 'left', $tr{'error messages'});
	print "<CLASS name='base'>$errormessage\n";
	print "&nbsp;</CLASS>\n";
	&closebox();
    }
}

showhttpheaders();
openpage($tr{'http antivirus'}, 1, '');
openbigbox('100%', 'left');

loadconfig();
my ($error, $errormessage) = save();
display_error($error, $errormessage);
display();
closebigbox();
closepage();

