#!/usr/bin/python

#
#        +-----------------------------------------------------------------------------+
#        | Endian Firewall                                                             |
#        +-----------------------------------------------------------------------------+
#        | Copyright (c) 2005-2006 Endian                                              |
#        |         Endian GmbH/Srl                                                     |
#        |         Bergweg 41 Via Monte                                                |
#        |         39057 Eppan/Appiano                                                 |
#        |         ITALIEN/ITALIA                                                      |
#        |         info@endian.it                                                      |
#        |                                                                             |
#        | This program is free software; you can redistribute it and/or               |
#        | modify it under the terms of the GNU General Public License                 |
#        | as published by the Free Software Foundation; either version 2              |
#        | of the License, or (at your option) any later version.                      |
#        |                                                                             |
#        | This program is distributed in the hope that it will be useful,             |
#        | but WITHOUT ANY WARRANTY; without even the implied warranty of              |
#        | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               |
#        | GNU General Public License for more details.                                |
#        |                                                                             |
#        | You should have received a copy of the GNU General Public License           |
#        | along with this program; if not, write to the Free Software                 |
#        | Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. |
#        | http://www.fsf.org/                                                         |
#        +-----------------------------------------------------------------------------+
#

from os import system as run
import os.path
import shutil
import sys
from commands import getoutput
from os import unlink
from os import remove
import signal
from Cheetah.Template import Template
from Cheetah.Tools import RecursiveNull

# catch the signal.
signal.signal(signal.SIGHUP, signal.SIG_DFL)


HAVP_ENABLE = "/var/efw/havp/enable"

CLAMAV_ENABLE = "/var/efw/clamav/enable_http"

VAR="/var/efw/havp/"
ETC="/etc/havp/"
HAVP_CONF = "havp.conf"

CLAMAV="/var/efw/clamav/"

PIDFILE='/var/run/havp/havp.pid'

def touch(filename):
    open(filename, 'a').close()

def getconfig(filename):
    """
    Reads a configuration file and returns a hash
    """
    config = {}
    try:
	for line in open(filename).xreadlines():
	    line = line.strip()
	    if line and line[0] != '#':
		line = line.split("=", 1)
		config[line[0]] = line[1]
    except:
	print "# Error reading config %s" % filename
	return {}
    return config

def write_config(TEMPLATE,CONF_FILE,VALUES):
    VALUES = alter_config(VALUES)

    null = RecursiveNull.RecursiveNull()
    t = Template(file=TEMPLATE, searchList=[VALUES, null])
    t2 = Template(t.respond(), searchList=[VALUES,null])

    content = t2.respond()
    file = open(CONF_FILE, 'w')
    file.write(content)
    file.close()

def getpid():
    try:
        fd = open(PIDFILE)
        pid = fd.readlines()[0].strip()
        fd.close()
        return pid
    except:
        return 0

def checkpid(pid):
    if pid == 0:
        return False
    if len(getoutput("ps h -o pid %s"%pid)) == 0:
        remove_pid()
        return False
    return True

def alter_config(VALUES):
    VALUES['MAXSCANSIZE'] = int(VALUES['MAXSCANSIZE']) * 1048576
    return VALUES

def writeHavpConfig():
    config_values = {}
    config_values.update(getconfig(CLAMAV+"/default/settings"))
    config_values.update(getconfig(VAR+"/default/settings"))
    config_values.update(getconfig(CLAMAV+"/settings"))
    config_values.update(getconfig(VAR+"/settings"))

    config_values['MAXSCANSIZE'] = int(config_values['MAXSCANSIZE']) * 1048576
    write_config(ETC+HAVP_CONF+".tmpl",ETC+HAVP_CONF,config_values)

def stop():
    run('/etc/init.d/havp condstop >/dev/null 2>&1')
    if os.path.exists("/usr/bin/monit"):
        run("monit unmonitor havp")
    if os.path.exists("/var/efw/monit/services/havp"):
        unlink("/var/efw/monit/services/havp")

def proxy_running():
    if len(getoutput("/bin/ps h -o pid -C squid")) > 0:
        return True
    return False

def start():
    writeHavpConfig()
    if not proxy_running():
        return
    run('/etc/init.d/havp start >/dev/null 2>&1')
    if os.path.exists("/usr/bin/monit"):
        run("monit monitor havp")
    if os.path.exists("/var/efw/monit/services/"):
        touch("/var/efw/monit/services/havp")


def restart():
    if os.access('/usr/local/bin/restartclamav', os.F_OK):
        run('/usr/local/bin/restartclamav  >/dev/null 2>&1')
    stop()
    if os.access(HAVP_ENABLE, os.F_OK):
        start()

def sync_clam():
    if os.access(HAVP_ENABLE, os.F_OK):
        touch(CLAMAV_ENABLE)
    else:
        if os.access(CLAMAV_ENABLE, os.F_OK):
            unlink(CLAMAV_ENABLE)

sync_clam()
restart()
sys.exit(0)
